         TITLE 'SOLOS/SYS8 INTEL FORMAT LOADER'
         ORG   0CA00H
;
;
;  ******** LOADER ROUTINES ********
;
;        FRAME     DESCRIPTION
;           0       COLON(":")
;         1-2       RECORD LENGTH(RL) IN HEX --LENGTH ZERO IS END OF FILE
;         3-6       STARTING ADDRESS (HIGH BYTE IN 3-4; LOW BYTE IN 5-6)
;         7-8       RECORD TYPE (ALWAYS ZERO OR ONE FOR END OF FILE)
; 9 TO 9+2*(RL)-1   2 FRAMES (HEX DIGITS) PER MEMORY WORD
; 9+2*(RL) TO 9+2*(RL)+1
;                   CHECKSUM (NEGATIVE OF ALL 8 BITS IN RECORD)
;
;  THE ROUTINE ENSURES THAT VALID INTEL HEX FORMAT DATA IS BEING LOADED BY
;  CHECKING FRAME 1 FOR A VALID HEX DIGIT. A <CR><LF> PAIR IS OUTPUT BEFORE
;  PROCESSING THE DATA FOR THE USER'S BENEFIT. A CHECK IS MADE FOR AN END OF FILE
;  ZERO LENGTH RECORD). THE LOADING ADDRESS FOR THE RECORD IS RETRIEVED AND DATA
;  IS READ IN. THE CHECKSUM IS CONTINUALLY ADDED AND FINALLY CHECKED WITH
;  THE SENT VALUE. ANY NONHEX CHARACTER IN THE RECORD (EXCLUDING A FRAME MARKER)
;  OR UNMET CHECKSUMS WILL RESULT IN AN ERROR MESSAGE AND A RETURN TO SOLOS.
;  "BYTE" DOES THE READING OF 2 VALID HEX ASCII CHARACTERS FROM THE DATA AND
;  DECODES THEM INTO BINARY IN REG "A". "RED0" READS FRAMES 0-8; "RED1" READS
;  THE DATA TO BE MEMORY LOADED AND CHECKSUM; "RED2" ALLOWS LOAD AND GO
;  EXECUTION ON THE END OF FILE BY SPECIFYING A NON PAGE ZERO ADDRESS
;  (OTHERWISE THE ADDRESS PAGE IS A ZERO AND A JUMP IS MADE BACK TO SOLOS).
;
;
LOAD1:   CALL  CRLF
         LXI   H,CSREQ
         CALL  SCRN
         LXI   H,CMSTR
LOAD2:   CALL  SINP
         JZ    LOAD2
         ANI   7FH
         CPI   ASNUL
         RZ
         CPI   ASDEL                      ;KLMSGE WILL KILL LINE..
         JNZ   LOAD3
         CALL  KLMSG
         JMP   LOAD1
LOAD3:   MOV   M,A
         MOV   B,A
         CALL  SOUT
         MOV   A,B
         CPI   ASCR
         JZ    LOAD4
         INX   H
         JMP   LOAD2
LOAD4:   CALL  CRLF
         MVI   A,1
         STA   IPORT
         STA   OPORT
         LXI   H,CMSTR
         CALL  SCRN
         CALL  CRLF
RED0:    CALL  MIN                     ;GET A CHAR FROM MODEM
         CPI   ':'
         JNZ   RED0                    ;LOOKING FOR A ':' RECORD MARK
         CALL  BYTE
         JZ    FINI
         MOH
SCONV    EQU             ;BLOCK LENGTH STORED IN E
         CALL  BYTE
         MOV   H,A
         CALL  BYTE
         MOV   L,A                     ;H & L NOV HAVE STARTING ADDRESS
         CALL  BYTE                    ;READ & IGNORE RECORD TYPE BYTE
RED1:    CALL  BYTE                    ;READ DATA & CHECKSUM INFO
         MOV   M,A                     ;STORE DATA IN SOL MEMORY
         INX   H                       ;BUMP MEMORY ADDRESS
         DCR   E                       ;DECREMENT BLOCK LENGTH COUNT
         JNZ   RED1                    ;LOOP IF NOT END OF BLOCK
         JMP   RED0                    ;READ NEXT BLOCK
BYTE:    CALL  KYCHK                   ;CHECK KEYBOARD
         CALL  INHEX                   ;GET AN ASCII HEX CHAR
         RLC
         RAL
         RAL
         RAL                           ;SHIFT TO BOTTOM OF ACCUMULATOR
         MOV   C,A                     ;STORE IN REG C
         CALL  INHEX                   ;GET AN ASCII HEX CHAR
         ADD   C
         RET
INHEX:   CALL  MIN                     ;GET A CHAR
         CALL  HCOV1                   ;CONVERT TO BINARY
         CPI   10H                     ;LEGAL VALUE ?
         RC                            ;RETURN IF SO
;
         LXI   H,ERMSG                 ;POINT TO ERROR MESSAGE
EM:      XRA   A
         STA   OPORT
         STA   IPORT
         CALL  SCRN
         CALL  CRLF
         JMP   COMND
MIN:     CALL  SSTAT                   ;GET A CHAR FROM MODEM
         JNZ   MIN2                    ;PROCESS CHAR IF AVAILABLE
         CALL  KYCHK                   ;CHECK KEYBOARD FOR ABORT
         JMP   MIN                     ;LOOP UNTIL CHAR AVAILABLE OR ABORTED
MIN2:    ANI   7FH                     ;STRIP PARITY BIT
         RET
;
KYCHK:   CALL  KSTAT
         RZ
         ANI   7FH
         CPI   ASNUL
         RNZ
         LXI   H,ABMSG
         JMP   EM
;
FINI:    LXI   H,FNMSG
         JMP   EM
;
SCRN:    MOV   B,M
         MOV   A,B
         CPI   ASCR
         RZ
         CALL  SOUT
         INX   H
         JMP   SCRN
;
KLMSG:   LXI   H,DLMSG
         CALL  SCRN
         CALL  CRLF
         RET
;
;    STORAGE FOR FILENAMES AND MESSAGES
;
CSREQ:   DB    'COMMAND STRING ?',ASCR
ERMSG:   DB    ASBEL,'LOAD ERROR',ASCR
DLMSG:   DB    ' *DEL*',ASCR
FNMSG:   DB    'LOAD COMPLETE.',ASCR
ABMSG:   DB    'LOAD ABORTED.',ASCR
CMSTR:   DB    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
;
;    SOLOS SYSTEM EQUATES
;
COMND    EQU   0C1C9H
KSTAT    EQU   0C02EH
CRLF     EQU   0C2F9H
PSCAN    EQU   0C310H
HCOV1    EQU   0C35DH
SOUT     EQU   0C019H
SINP     EQU   0C01FH
SSTAT    EQU   0C042H
IPORT    EQU   0C806H
OPORT    EQU   0C807H
;
;    CHARACTER DEFINITIONS
;
ASNUL    EQU   00H
ASDEL    EQU   7FH
ASCR     EQU   0DH
ASBEL    EQU   07H
         END
